@@ -3,6 +3,7 @@ import json
 import logging
 from urllib.parse import parse_qsl, urlencode, urlparse
 
+from django import http
 from django.contrib.auth.mixins import LoginRequiredMixin
 from django.contrib.auth.views import redirect_to_login
 from django.http import HttpResponse
@@ -12,6 +13,9 @@ from django.utils.decorators import method_decorator
 from django.views.decorators.csrf import csrf_exempt
 from django.views.decorators.debug import sensitive_post_parameters
 from django.views.generic import FormView, View
+from oauthlib.oauth2.rfc8628 import errors as rfc8628_errors
+
+from oauth2_provider.models import DeviceGrant
 
 from ..compat import login_not_required
 from ..exceptions import OAuthToolkitError
@@ -290,10 +294,13 @@ class TokenView(OAuthLibMixin, View):
     * Authorization code
     * Password
     * Client credentials
+    * Device code flow (specifically for the device polling stage)
     """
 
-    @method_decorator(sensitive_post_parameters("password"))
-    def post(self, request, *args, **kwargs):
+    @method_decorator(sensitive_post_parameters("password", "client_secret"))
+    def authorization_flow_token_response(
+        self, request: http.HttpRequest, *args, **kwargs
+    ) -> http.HttpResponse:
         url, headers, body, status = self.create_token_response(request)
         if status == 200:
             access_token = json.loads(body).get("access_token")
@@ -307,6 +314,68 @@ class TokenView(OAuthLibMixin, View):
             response[k] = v
         return response
 
+    def device_flow_token_response(
+        self, request: http.HttpRequest, device_code: str, *args, **kwargs
+    ) -> http.HttpResponse:
+        try:
+            device = DeviceGrant.objects.get(device_code=device_code)
+        except DeviceGrant.DoesNotExist:
+            # The RFC does not mention what to return when the device is not found,
+            # but to keep it consistent with the other errors, we return the error
+            # in json format with an "error" key and the value formatted in the same
+            # way.
+            return http.HttpResponseNotFound(
+                content='{"error": "device_not_found"}',
+                content_type="application/json",
+            )
+
+        # Here we are returning the errors according to
+        # https://datatracker.ietf.org/doc/html/rfc8628#section-3.5
+        # TODO: "slow_down" error (essentially rate-limiting).
+        if device.status == device.AUTHORIZATION_PENDING:
+            error = rfc8628_errors.AuthorizationPendingError()
+        elif device.status == device.DENIED:
+            error = rfc8628_errors.AccessDenied()
+        elif device.status == device.EXPIRED:
+            error = rfc8628_errors.ExpiredTokenError()
+        elif device.status != device.AUTHORIZED:
+            # It's technically impossible to get here because we've exhausted
+            # all the possible values for status. However, it does act as a
+            # reminder for developers when they add, in the future, new values
+            # (such as slow_down) that they must handle here.
+            return http.HttpResponseServerError(
+                content='{"error": "internal_error"}',
+                content_type="application/json",
+            )
+        else:
+            # AUTHORIZED is the only accepted state, anything else is
+            # rejected.
+            error = None
+
+        if error:
+            return http.HttpResponse(
+                content=error.json,
+                status=error.status_code,
+                content_type="application/json",
+            )
+
+        url, headers, body, status = self.create_token_response(request)
+        response = http.JsonResponse(data=json.loads(body), status=status)
+
+        if status != 200:
+            return response
+
+        for k, v in headers.items():
+            response[k] = v
+
+        return response
+
+    def post(self, request: http.HttpRequest, *args, **kwargs) -> http.HttpResponse:
+        params = request.POST
+        if params.get("grant_type") == "urn:ietf:params:oauth:grant-type:device_code":
+            return self.device_flow_token_response(request, params["device_code"])
+        return self.authorization_flow_token_response(request)
+
 
 @method_decorator(csrf_exempt, name="dispatch")
 @method_decorator(login_not_required, name="dispatch")
