@@ -2,12 +2,13 @@ import json
 
 import pytest
 from django.contrib.auth import get_user_model
-from django.test import RequestFactory, TestCase
+from django.test import RequestFactory
 from django.urls import reverse
 
 from oauth2_provider.models import get_application_model
 from oauth2_provider.views import ProtectedResourceView
 
+from .common_testing import OAuth2ProviderTestCase as TestCase
 from .utils import get_basic_auth_header
 
 
