@@ -1,12 +1,14 @@
 import datetime
 
 from django.contrib.auth import get_user_model
-from django.test import RequestFactory, TestCase
+from django.test import RequestFactory
 from django.urls import reverse
 from django.utils import timezone
 
 from oauth2_provider.models import get_access_token_model, get_application_model, get_refresh_token_model
 
+from .common_testing import OAuth2ProviderTestCase as TestCase
+
 
 Application = get_application_model()
 AccessToken = get_access_token_model()
@@ -53,7 +55,7 @@ class TestRevocationView(BaseTest):
         response = self.client.post(url, data=data)
         self.assertEqual(response.status_code, 200)
         self.assertEqual(response.content, b"")
-        self.assertFalse(AccessToken.objects.filter(id=tok.id).exists())
+        self.assertFalse(AccessToken.objects.filter(pk=tok.pk).exists())
 
     def test_revoke_access_token_public(self):
         public_app = Application(
@@ -101,7 +103,7 @@ class TestRevocationView(BaseTest):
         url = reverse("oauth2_provider:revoke-token")
         response = self.client.post(url, data=data)
         self.assertEqual(response.status_code, 200)
-        self.assertFalse(AccessToken.objects.filter(id=tok.id).exists())
+        self.assertFalse(AccessToken.objects.filter(pk=tok.pk).exists())
 
     def test_revoke_access_token_with_invalid_hint(self):
         tok = AccessToken.objects.create(
@@ -123,7 +125,7 @@ class TestRevocationView(BaseTest):
         url = reverse("oauth2_provider:revoke-token")
         response = self.client.post(url, data=data)
         self.assertEqual(response.status_code, 200)
-        self.assertFalse(AccessToken.objects.filter(id=tok.id).exists())
+        self.assertFalse(AccessToken.objects.filter(pk=tok.pk).exists())
 
     def test_revoke_refresh_token(self):
         tok = AccessToken.objects.create(
@@ -146,9 +148,9 @@ class TestRevocationView(BaseTest):
         url = reverse("oauth2_provider:revoke-token")
         response = self.client.post(url, data=data)
         self.assertEqual(response.status_code, 200)
-        refresh_token = RefreshToken.objects.filter(id=rtok.id).first()
+        refresh_token = RefreshToken.objects.filter(pk=rtok.pk).first()
         self.assertIsNotNone(refresh_token.revoked)
-        self.assertFalse(AccessToken.objects.filter(id=rtok.access_token.id).exists())
+        self.assertFalse(AccessToken.objects.filter(pk=rtok.access_token.pk).exists())
 
     def test_revoke_refresh_token_with_revoked_access_token(self):
         tok = AccessToken.objects.create(
@@ -172,8 +174,8 @@ class TestRevocationView(BaseTest):
             response = self.client.post(url, data=data)
             self.assertEqual(response.status_code, 200)
 
-        self.assertFalse(AccessToken.objects.filter(id=tok.id).exists())
-        refresh_token = RefreshToken.objects.filter(id=rtok.id).first()
+        self.assertFalse(AccessToken.objects.filter(pk=tok.pk).exists())
+        refresh_token = RefreshToken.objects.filter(pk=rtok.pk).first()
         self.assertIsNotNone(refresh_token.revoked)
 
     def test_revoke_token_with_wrong_hint(self):
@@ -202,4 +204,4 @@ class TestRevocationView(BaseTest):
         url = reverse("oauth2_provider:revoke-token")
         response = self.client.post(url, data=data)
         self.assertEqual(response.status_code, 200)
-        self.assertFalse(AccessToken.objects.filter(id=tok.id).exists())
+        self.assertFalse(AccessToken.objects.filter(pk=tok.pk).exists())
